

# Control Technology Corporation
# 25 South Street, Suite A2
# Hopkinton, MA 01748
#
# Simple test program to show how to read a controller register using
# the Ctccom32v2.dll. If using 64 bit Python the 64 bit dll must be used.
# If using 32 bit Python the 32 bit dll must be used.
#
# Standalone .exe can be created by using pyinstaller
# pyinstaller CTCComPythonTest.py --onefile
# The dist subfolder will contain the .exe executable file for standalone distribution.
# Note that the Ctccom32v2.dll location must match the path of the program.
#
import ctypes
import socket
import sys
import time
import struct

from ctypes import *

class Ctccomm32v2:
    #  PURPOSE:
    #    Contains declarations used for Ctccom32v2 DLL.
    #
    # constants for binary protocol
    CT_ASCII   = int(10)
    CT_REQUEST   = int(20)
    CT_REPLY   = int(21)
    CT_ACK   = int(100)
    CT_NAK   = int(101)
    CT_RANGE   = int(102)
    CT_VALUE   = int(104)
    # function return values
    FAILURE = 0
    SUCCESS  = 1
    
    ERROR_UNKNOWN  = 255
    ERROR_WIN_TIMER  = 246
    ERROR_NET_TIMEOUT   = 236
    ERROR_NET_NOTREADY   = 235
    ERROR_CT_RANGE   = 216
    ERROR_BAD_CHECKSUM   = 215
    ERROR_BAD_LENGTH   = 214
    ERROR_BAD_RESPONSE   = 206
    ERROR_NOT_CT_PROTOCOL   = 205
    ERROR_UNSUPP_REMOTE_OPER   = 196
    ERROR_UNKNOWN_REM_NODE   = 195
    ERROR_REMOTE_ACCESS   = 194
    ERROR_OBJECT_FILE_NOT_FOUND   = 186
    ERROR_FILE_WRONG_ARCH   = 185
    ERROR_FILE_NOT_OBJECT   = 184
    ERROR_BAD_OBJECT_RECORD   = 183
    ERROR_BAD_OBJECT_LENGTH   = 182
    ERROR_BAD_POSITION   = 181
    ERROR_EOF   = 180
    ERROR_BAD_PORT   = 179
    ERROR_NET_IN_USE   = 178
    ERROR_SERIAL_TIMEOUT   = 170
    ERROR_WARNING   = 169
    ERROR_ILLEGAL_BAUD_RATE   = 168
    ERROR_ILLEGAL_PARITY_SETTING   = 167
    ERROR_ILLEGAL_WORD_LENGTH   = 166
    ERROR_ILLEGAL_STOP_BITS   = 16
    ERROR_ERROR   = 160
    ERROR_PORT_NOT_FOUND   = 159
    ERROR_PORT_IN_USE   = 158
    # Constants for port access types
    NO_PORT   = 0
    SERIAL   = 1
    CTCNET   = 2
    CTCUDP   = 3
    CTCTCP   = 4

def dottedQuadToNum(ip):
    "convert decimal dotted quad string to long integer"
    return struct.unpack('L',socket.inet_aton(ip))[0]

# Load the CTC dll that is in the directory above the python code and/or .exe if built with pyinstaller. 
# If using 64 bit Python it must be a 64 bit dll else use the 32 bit dll.  This example is 64 bit.
# Change the location as desire by changing the path to Ctccom32v2.dll.
CTC_dll = ctypes.CDLL("..\\Ctccom32v2.dll")

#   Some DLL functions written in 'C'/C++ that are expose are defined below
#   // network access functions 
#    [DllImport("Ctccom32v2.dll")] public static extern int CtOpenConnection 
#        (int ConnectID, int CommPort, int Address);
CTC_dll.CtOpenConnection.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int32]
CTC_dll.CtOpenConnection.restype = ctypes.c_int32

#    [DllImport("Ctccom32v2.dll")] public static extern int CtInitConnection
#        (int ConnectID, int BaudRate, byte Parity, byte DataBits, byte StopBits);
CTC_dll.CtInitConnection.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_byte, ctypes.c_byte, ctypes.c_byte]
CTC_dll.CtInitConnection.restype = ctypes.c_int32

#    [DllImport("Ctccom32v2.dll")] public static extern int CtCloseConnection 
#        (int ConnectID);
CTC_dll.CtCloseConnection.argtypes = [ctypes.c_int32]
CTC_dll.CtCloseConnection.restype = ctypes.c_int32

#    [DllImport("Ctccom32v2.dll")] public static extern int CtSetConnectionTimeout
#        (int ConnectID, int MilliSeconds);
CTC_dll.CtSetConnectionTimeout.argtypes = [ctypes.c_int32, ctypes.c_int32]
CTC_dll.CtSetConnectionTimeout.restype = ctypes.c_int32

#    // register functions
#    [DllImport("Ctccom32v2.dll")] public static extern int CtGetRegister 
#        (int ConnectID, int RegAddress, int *RegValue);
CTC_dll.CtGetRegister.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.POINTER(ctypes.c_ulong)]
CTC_dll.CtGetRegister.restype = ctypes.c_int32

#    [DllImport("Ctccom32v2.dll")] public static extern int CtPutRegister
#        (int ConnectID, int RegAddress, int RegValue);
CTC_dll.CtPutRegister.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_ulong]
CTC_dll.CtPutRegister.restype = ctypes.c_int32

# Set how we want to access the controller, uncomment one:
#accessType = "UDP"
accessType = "TCP"
#accessType = "SERIAL"
LOOPIT = 0
serialPort = 1  # for COMM1 if accessType set to SERIAL
serialBaud = 19200 # Serial port baud rate if accessType set to SERIAL

controllerIPAddress = int(dottedQuadToNum("172.16.0.199"))  # Set to controller IP address and pass as little endian

if (accessType == "UDP"):
    # It's a UDP connection...  Note UDP is connectionless so check status returned when read a register
    CTconnection = CTC_dll.CtOpenConnection(Ctccomm32v2.CTCUDP, 0, controllerIPAddress);
elif (accessType == "TCP"):
    # It's a TCP connection
    CTconnection = CTC_dll.CtOpenConnection(Ctccomm32v2.CTCTCP, 0, controllerIPAddress);
else:
    # It's a serial connection
    CTconnection = CTC_dll.CtOpenConnection(Ctccomm32v2.SERIAL, serialPort, 1);
    CTC_dll.CtInitConnection(CTconnection, serialBaud, Ctccomm32v2.NO_PARITY, Ctccomm32v2.EIGHT_DATA_BITS, Ctccomm32v2.ONE_STOP_BITS);
    
  
# Check our connection status
if CTconnection == Ctccomm32v2.FAILURE:
    sys.exit("Did not connect to the controller.")
else:
    print("Connected to controller...")
    counter = ctypes.c_ulong
    counter.value = 0
while LOOPIT == 0:
    regVal = ctypes.c_ulong()
    status = CTC_dll.CtGetRegister(CTconnection, 13002, ctypes.byref(regVal))  # Read the controller millisecond timer register
    if status != Ctccomm32v2.SUCCESS:
        sys.exit("If Using UDP did not connect to the controller otherwise connection lost.")
        
    print("Register 13002 = " + str(regVal.value));
    time.sleep(.1)  # pause about 100 milliseconds and then read the controller 1 ms tick clock register again
    status = CTC_dll.CtGetRegister(CTconnection, 13002, ctypes.byref(regVal))
    if status != Ctccomm32v2.SUCCESS:
        sys.exit("Connection lost, aborting.")
    status = CTC_dll.CtPutRegister(CTconnection, 5, counter.value)      
    counter.value = counter.value + 1
    # Successful to list contents to console
    print("Register 13002 = " + str(regVal.value));

# Close our connection to the controller and then exit the program.
CTC_dll.CtCloseConnection(CTconnection)
sys.exit("Python CTCCom DLL interface test passed.")

